# Library LibNAC::Tools
# Holds optional tools to facilitate in the administration of a circe NAC 
# system.
# Author: Hao Bryan Cheng (hbcheng@berkeley.edu)
# Tested with Perl 5.10.1

package LibNAC::Tools;

use LibNAC;
use Net::IP::Match::XS;

# Constant declarations
use constant {
	ERR_INVALID_NETWORK => -1,
	ERR_INVALID_NETMASK => -2,
	ERR_INVALID_ROUTER => -3,
	ERR_ROUTER_NOT_IN_SUBNET => -4,
	ERR_INVALID_IP_IN_RANGE => -5,
	ERR_MALFORMED_CLIENT_RANGE => -6,
	ERR_IF_MISSNG => -7,
};

our $valid_ip_regex= '^((1?[0-9][0-9]?|2([0-4][0-9]|5[0-5]))\.){3}(1?[0-9][0-9]?|2([0-4][0-9]|5[0-5]))$';

# Script to validate nac.conf. Provides basic syntax checking and
# some primitive option validation
sub nac_conf_validate {

    # grab a configration hash to work with
    my %config 		= LibNAC::nac_readconf();

	my @required	= qw(LOCKFILE LOCKFILE_STORE_PATH IPFW_BEGIN_SCRIPT 
		PIPE_CIRCE_IN PIPE_CIRCE_OUT NG_CIRCE_IN NG_CIRCE_OUT ASSOCIATED_NETS
		ONATPREFIX INATPREFIX NAME_PREFIX DEFAULT_CONTEXT CONTEXTS REDIRECTS
		PUBLIC_TABLES PRIVATE_TABLES CIRCE_CLIENTS_IF NAC_PUBLIC_NETS 
		NAC_PRIVATE_NETS AUTH_HOST_MAP);

	my @missing_fatal;

    ## Fatal errors
    # Check for required options
    # LOCKFILE, LOCKFILE_STORE_PATH, IPFW_BEGIN_SCRIPT, TABLE*, context to 
    # table bindings, PIPE_CIRCE_{IN,OUT}, NG_CIRCE_{IN,OUT}, ASSOCIATED_NETS,
    # {O,I}NATPREFIX, NAME_PREFIX, NAME*, *_PAGE, DEFAULT_CONTEXT,
    # CONTEXTS, REDIRECTS, PUBLIC_TABLES, PRIVATE_TABLES, CIRCE_CLIENTS_IF,
    # NAC_PUBLIC_NETS, NAC_PRIVATE_NETS, AUTH_HOST_MAP
	
	foreach my $req_option (@required) {
		unless ($config{"$req_option"}) {
			push(@missing_fatal,$req_option);
		}
	}

	nac_make_human_readable(\@missing_fatal);


    # Check syntax of select options
	my @NAC_PUBLIC_NETS = split("\n",$config{"NAC_PUBLIC_NETS"});
	my @NAC_PRIVATE_NETS = split("\n",$config{"NAC_PRIVATE_NETS"});
	my @nac_nets;
	push (@nac_nets,@NAC_PRIVATE_NETS);
	push (@nac_nets,@NAC_PUBLIC_NETS);
	chomp (@nac_nets);
	foreach my $net (@nac_nets) {
		$net =~ s/^[\s]+//;
		unless ($net_str =~ m/^[\s]*$/) {
			# Check NAC_{PUBLIC,PRIVATE}_NETS for syntax
			nac_net_validate($net);
			# Check that subnet declarations for NETS are valid and exist within
			# ASSOCIATED_NETS map
			my ($subnet) = LibNAC::nac_subnet($net);
			my ($oct1,$oct2,undef,undef) = split(/\./,$subnet);
			my $id	= LibNAC::nac_associated_net_id($subnet,\%config);
			if ($id == 99) {
				print ("Error: subnet $subnet not in any ASSOCIATED_NETS\n");
			}
		}   
	}
    # Check length of ONAT and INAT PREFIX
	#
	my ($onatprefix) 	= $config{"ONATPREFIX"};
	my ($inatprefix) 	= $config("INATPREFIX"};
	
	unless (length($onatprefix) == 1 && length($inatprefix) == 1) {
		print ("Invalid prefix length for ONATPREFIX or INATPREFIX:\n");
		print ("The specified length for these options is one character.\n");
	
    # Check table numbers correspond to actual tables
    # Check interfaces CIRCE_CLIENTS_IF and NAC_*_NETS exist on system

    ## Non-fatal errors
    # Check CIRCE_CLIENTS ip exists and is valid. This is usually non-fatal
    # but could be fatal if CIRCE_CLIENTS_IF has multiple tagged ip addrs
    # Check boolean validitiy for options
    # Check deprecated options for removal or renaming
    #
}

sub nac_net_validate {
	my $net_str		= shift;
	#print ("nac_net_validate got net string $net_str\n");
	my %net_hash 	= LibNAC::nac_net_parse($net_str,"all");

	my $error_network;
	my $error_netmask;
	my $error_router;
	my $error_clientrange;
	my $error_interface;

	## Check the subnet declaration.
	my ($network,$netmask)	= split("/",$net_hash{"subnet"});
	my $subnet	= $net_hash{"subnet"};
	$network =~ s/^[\s]*//;
	$subnet =~ s/^[\s]*//;
	# network should be a valid ip address
	unless ($network =~ /($valid_ip_regex)/) {
		print ("Network: $network \n");
		print ("Network declaration in NET-formatted string $net_str is not a valid ip address\n");
		$error_network=ERR_INVALID_NETWORK;
	}
	# Netmask should be an integer less than 32. Although if its less than 12 or so,
	# we should probably warn.
	unless ($netmask == int($netmask) && $netmask < 32) {
		print ("Network declaration in NET-formatted string $net_str requires valid CIDR netmask\n");
		$error_netmask=ERR_INVALID_NETMASK;
	}
	unless ($netmask >= 12 ) {
		print ("Possible typo in NET-formatted string $net_str: very large ip address block declared\n");
	}

	## Check the router declaration
	my ($router) 	= $net_hash{"router"};
	# router should be a valid ip address.
	unless ($router =~ /($valid_ip_regex)/) {
		print ("Router declaration in NET-formatted string $net_str is not a valid ip address\n");
		$error_router = ERR_INVALID_ROUTER;
	} elsif (!($error_network ~~ ERR_INVALID_NETWORK)) {
		unless (LibNAC::nac_ip_in_subnet($router, $subnet)) {
			# The router should exist within the specified subnet
			print ("Error in NET-formatted string $net_str: router $router not in subnet $subnet\n");
			$error_router = ERR_ROUTER_NOT_IN_SUBNET;
		}
	}

	## Check the client range
	# Note: The syntax technically allows multiple client range declarations 
	# but LibNAC doesn't parse this situation properly yet. We'll warn about it
	# here
	my $clientrange	= $net_hash{"clientrange"};
	my @clientranges = split(",",$clientrange);
	if ($#clientranges >= 1) {
		print ("Detected multiple client range declarations in NET-formatted string $net_str.\n");
		print ("This behavior is not yet supported by circe. You may workaround by creating additional NET-formatted strings for each client range.\n");
	}
	# Make sure the range makes sense and that both the low and high end are within
	# the specified subnet
	RANGES:
	foreach my $range (@clientranges) {
		my ($low,$high)	= split('-',$range);
		unless ($low =~ /($valid_ip_regex)/ && $high =~ /($valid_ip_regex)/) {
			print ("Invalid ip address specified in client range statement of NET-formatted string $net_str\n");
			$error_clientrange = ERR_INVALID_IP_IN_RANGE;
			last RANGES;
		}

		$low =~ s/^[\s]*//;
		$high =~ s/^[\s]*//;
		my $low_dec		= LibNAC::nac_ip_to_num($low);
		my $high_dec	= LibNAC::nac_ip_to_num($high);
		unless ($high_dec > $low_dec) {
			print ("Invalid client range specified in NET-formatted string $net_str: High end of range was lower than or equal to low end.\n");
			$error_clientrange = ERR_MALFORMED_CLIENT_RANGE;
			last RANGES;
		} 
		unless (LibNAC::nac_ip_in_subnet($high,$subnet)) {
			print ("Higher bound of client range $high exceeds subnet range in NET-formatted string $net_str.\n");
			$error_clientrange = ERR_MALFORMED_CLIENT_RANGE;
		}
		unless (LibNAC::nac_ip_in_subnet($low,$subnet)) {
			print ("Lower bound of client range $low exceeds subnet range in NET-formatted string $net_str.\n");
			$error_clientrange = ERR_MALFORMED_CLIENT_RANGE;
		}
	}

	## Check the interface
	my $if	= $net_hash{"interface"};
	# First, make sure it exists
	$ifconfig = `ifconfig $if inet`;
	if ($ifconfig =~ /does not exist$/) {
		print ("Interface $if in NET-formatted string $net_str does not exist on system.\n");
		print ("Check your network configuration and nac.conf for errors.\n");
		$error_interface = ERR_IF_MISSING;
	}
	# Now check that the ip address assigned to the interface is inside the 
	# subnet declared
	my @ip_addresses;
	unless ($error_interface ~~ ERR_IF_MISSING) {
		my $if_has_ip_in_subnet = 0;
		@ifconfig_inets	= split("\n",$ifconfig);
		foreach my $inet (@ifconfig_inets) {
			if ($inet =~ /^[\s]+inet/) {
				my (undef,$ip,undef) = split(" ",$inet);
				push(@ip_addresses,$ip);
			}
		}
		foreach my $ip (@ip_addresses) {
			if (LibNAC::nac_ip_in_subnet($ip,$subnet)) {
				$if_has_ip_in_subnet=1;
			}
		}
		unless ($if_has_ip_in_subnet) {
			print ("Interface $if does not have an ip adddress in subnet $subnet.\n");
			print ("Check network configuration and NET-formatted string $net_str.\n");
		}
	}

}

sub nac_make_human_readable {
	$fatal_aref	= shift;
	@fatal		= @$fatal_aref;

	foreach $missing_option (@fatal) {
		print ("FATAL: Nac.conf option $missing_option is missing.\n");
		print ("This option is required for successful circe operation.\n");
	}
	return 1;
}

1;

