# Package LoadTest::Management
# 
# Perl module within the LoadTest library containing functions for the 
# management of client scripts. Includes functionality for spawning 
# new clients and monitoring system response and performance.
#

package LoadTest::Management;

use Sys::Syslog qw(:DEFAULT setlogsock);
use lib '/home/bcheng/';
use LoadTest::Client;
use Time::HiRes qw(time);
use constant {
	REPORTDIR => "/home/bcheng/reports/",
}; 


# Main management function. Initiates a test using the settings found in the
# config file.
sub management_begin_test {
	my ($config_path, $duration) = @_;
	$arefClients = management_parse_config($config_path);

	my @clients = @$arefClients;
	my @pids;

	foreach my $client (@clients) {
		@client_info = @$client;
		@net = @{$client_info[0]};
		my $pid = management_spawn_client($client,$duration);
		push (@pids,$pid);
	}

	sleep($duration);

	foreach my $pid (@pids) {
		while (management_is_running($pid)) { 
			print "$pid is still running, sleeping\n";
			sleep(30); 
		}# do nothing
		# This halts execution of the main managment thread until
		# all children have safely exited
	}

	management_generate_summary($#pids,$duration,\@clients);

}


### Spawn a new client with the included parameters
sub management_spawn_client {
	my ($client,$duration) = @_;
	my $pid = fork();
	if (not defined $pid) {
		management_log ("Insufficient Resources to fork.","err");
		return 0;
	} elsif ($pid == 0) {
		# Child Case
		my $start = time();
		my $elapsed = time() - $start;

		while ($elapsed < $duration) {
			#my $remaining = $duration - $elapsed;
			@client_info = @$client;
			@net = @{$client_info[0]};
			print "This is the child! Spawning client with @net\n";
			print "Elapsed: $elapsed, Duration $duration\n";
			my $overall_start = time();
			my $href_metrics = LoadTest::Client::client(@$client);
			my $overall = time() - $overall_start;
			$href_metrics->{"overall"}= $overall;
			management_record_metrics($net[0],$href_metrics);
			$elapsed = time() - $start;
		}
		exit;
	} else {
		# Parent Case
		if (management_is_running($pid)) {
			management_log("Child successfully forked","err");
			return $pid;
		} else {
			management_log("Unsuccessful fork attempt.","err");
			return 0;
		}
	}
}


sub management_is_running {
	my $pid = shift;

	my $pslines     = `/bin/ps $pid | /usr/bin/wc -l`;
	$pslines        -= 1;
	return $pslines;
}

sub management_parse_config {
	my $path = shift;

	open (CONF, $path) 
		or management_log("Error: Can't find config file","crit");
	my @clients;

	while ($line = <CONF> ) {
		#Each line represents a single client.
		# Client information comes in the order private ip, public ip, mac,
		# refresh count, failed login count, successful login count, session 
		# length, username, password.

		unless ($line eq "\n" || $line =~ m/^\s*#/) {
			my ($private_ip, $public_ip, $mac, $refresh_count, 
				$failed_login_count, $login_count, $session_length,  $username,
				$password) = split (" ", $line);

			my @client_nets = ($private_ip, $public_ip, $mac);
			my @client_login = ($username, $password);

			my @client = (\@client_nets, $refresh_count, $failed_login_count,
				$login_count, \@client_login, $session_length);
			push (@clients,\@client); 
		}
	}

	return \@clients;
}

sub management_record_metrics {
	my ($ip, $href_metrics)  = @_;
	my %metrics = %$href_metrics;
	my $date = `date`;
	open(RECORD, ">>" . REPORTDIR . "$ip");
	my $aref_raw_refresh = $href_metrics->{"raw_refresh"};
	my $aref_raw_failed_logon = $href_metrics->{"raw_failed_logon"};
	my $aref_raw_logon = $href_metrics->{"raw_logon"};
	my @raw_refresh = @$aref_raw_refresh;
	my @raw_failed_logon = @$aref_raw_failed_logon;
	my @raw_logon = @$aref_raw_logon;
	
	print RECORD "Performance record for $ip run ending $date\n";
	print RECORD "Overall time for run: " . $href_metrics->{"overall"} . "\n";
	print RECORD "Refresh Metrics:\n";
	print RECORD "\tRaw Refresh: @raw_refresh \n";
	print RECORD "\tAverage Refresh Time: " . $href_metrics->{"avg_refresh"}
		. "\n";
	print RECORD "\tRefresh Standard Deviation: " . 
		$href_metrics->{"stddev_refresh"} . "\n";
	print RECORD "\tRefresh Time Range: " . $href_metrics->{"range_refresh"} 
		. "\n";
	
	print RECORD "Failed Logon Metrics:\n";
	print RECORD "\tAverage Failed Logon Time: " . 
		$href_metrics->{"avg_failed_logon"}	. "\n";
	print RECORD "\tFailed Logon Standard Deviation: " . 
		$href_metrics->{"stddev_failed_logon"} . "\n";
	print RECORD "\tFailed Logon Time Range: " . 
		$href_metrics->{"range_failed_logon"} . "\n";

	print RECORD "Successful Logon Metrics:\n";
	print RECORD "\tAverage Logon Time: " . 
		$href_metrics->{"avg_logon"}	. "\n";
	print RECORD "\tLogon Standard Deviation: " . 
		$href_metrics->{"stddev_logon"} . "\n";
	print RECORD "\tLogon Time Range: " . 
		$href_metrics->{"range_logon"} . "\n";

	if ($href_metrics->{"logon_errors"}) {
		print RECORD "NOTE: An error occured during logon: ";
		print RECORD $href_metrics->{"logon_errors"} . "\n";
	}
	if ($href_metrics->{"logout_errors"}) { 
		print RECORD "NOTE: An error occured during logout: ";
		print RECORD $href_metrics->{"logout_errors"} . "\n";
	}

	print RECORD "End metrics for run.\n";
	close(RECORD);

	open(SHARED,">>" . REPORTDIR . "shared_refresh");
	until (flock(SHARED,2)) {
		close(SHARED);
		open(SHARED,">>" . REPORTDIR . "shared_refresh");
	}
	print SHARED "@raw_refresh ";
	close(SHARED);

	open(SHARED,">>" . REPORTDIR . "shared_failed_logon");
	until (flock(SHARED,2)) {
		close(SHARED);
		open(SHARED,">>" . REPORTDIR . "shared_failed_logon");
	}
	print SHARED "@raw_failed_logon ";
	close(SHARED);
	
	open(SHARED,">>" . REPORTDIR . "shared_logon");
	until (flock(SHARED,2)) {
		close(SHARED);
		open(SHARED,">>" . REPORTDIR . "shared_logon");
	}
	print SHARED "@raw_logon ";
	close(SHARED);
	return 1;
}

# Produce the overall executive summary for the Load test
sub management_generate_summary {
	my ($count,$duration,$aref_clients) = @_;
	$count += 1;
	my $date = `date`;
	my @clients = @$aref_clients;
	my @refresh_times;

	my $refresh = Statistics::Descriptive::Full->new();
	my $failed_logon = Statistics::Descriptive::Full->new();
	my $logon = Statistics::Descriptive::Full->new();

	open (SUMMARY, ">>" . REPORTDIR . "summary");
	open (REFRESH, REPORTDIR . "shared_refresh");
	open (FAILEDLOGON, REPORTDIR . "shared_failed_logon");
	open (LOGON, REPORTDIR . "shared_logon");

	while (my $line = <REFRESH>) {
		@refresh_times = split(" ",$line);
		foreach my $time (@refresh_times) {
			$refresh->add_data($time);
		}
	}

	while (my $line = <FAILEDLOGON>) {
		@failed_logon_times = split(" ",$line);
		foreach my $time (@failed_logon_times) {
			$failed_logon->add_data($time);
		}
	}
			
	while (my $line = <LOGON>) {
		@logon_times = split(" ",$line);
		foreach my $time (@logon_times) {
			$logon->add_data($time);
		}
	}

	close(REFRESH);
	close(FAILEDLOGON);
	close(LOGON);

	print SUMMARY "Summary and statistics for load test run ending at $date\n";
	print SUMMARY "Test length: $duration seconds, spawned $count clients.\n";
	print SUMMARY "Client List:\n";
	foreach my $aref_client (@clients) {
		my @client = @$aref_client;
		my @net = @{$client[0]};
		print SUMMARY "@net\n";
	}
	print SUMMARY "\n=====BEGIN DATA=====\n";
	print SUMMARY "Refresh Statistics:\n";
	print SUMMARY "Total of " . $refresh->count . " samples taken.\n";
	print SUMMARY "\tAverage Refresh time: " . $refresh->mean() . " seconds\n";
	print SUMMARY "\tMinimum Refresh time: " . $refresh->min() . " seconds\n";
	print SUMMARY "\tMaximum Refresh time: " . $refresh->max() . " seconds\n";
	print SUMMARY "\tRefresh time standard deviation: " . 
		$refresh->standard_deviation() . " seconds\n";
		
	print SUMMARY "Failed Logon Statistics:\n";
	print SUMMARY "Total of " . $failed_logon->count . " samples taken.\n";
	print SUMMARY "\tAverage Failed Logon time: " . $failed_logon->mean() 
		. " seconds\n";
	print SUMMARY "\tMinimum Failed Logon time: " . $failed_logon->min()
		. " seconds\n";
	print SUMMARY "\tMaximum Failed Logon time: " . $failed_logon->max() 
		. " seconds\n";
	print SUMMARY "\tFailed Logon time standard deviation: " . 
		$failed_logon->standard_deviation() . " seconds\n";

	print SUMMARY "Logon Statistics:\n";
	print SUMMARY "Total of " . $logon->count . " samples taken.\n";
	print SUMMARY "\tAverage Logon time: " . $logon->mean() . " seconds\n";
	print SUMMARY "\tMinimum Logon time: " . $logon->min() . " seconds\n";
	print SUMMARY "\tMaximum Logon time: " . $logon->max() . " seconds\n";
	print SUMMARY "\tLogon time standard deviation: " . 
		$logon->standard_deviation() . " seconds\n";
	
	print SUMMARY "\n=====END DATA=====\n";
	print SUMMARY "Raw data is available in the shared_refresh, " .
		"shared_failed_logon, and shared_logon files.\n";
	close(SUMMARY);

}
 
sub management_log {
	my ($line, $priority) = @_;

	setlogsock('unix');
	openlog("load-test",'','local3');
	syslog($priority, $line);
	closelog;
	return 1;
}

1;
