# Module LoadTest::Server
#
# This module consists of the libraries for the server-side elements of
# the load testing scripts. This library provides functions for monitoring
# system load and performance and parsing relevant system logs.

package LoadTest::Server;

use Time::HiRes qw(time);
use LibNAC;
use constant {
	OUTPUT => "/home/bcheng/server_output",
	WORKING => "/home/bcheng/server/",
};

sub server_load_check {
	# Make a local copy of the circe log
	system("cp /var/log/circe.log " . WORKING . "circe.log.current") ;
	my (undef, $ncpu) = split(":", `sysctl hw.ncpu`);

	my %report;

	my @uptime  = split(",",`uptime`);

	my $load_one =  $uptime[3];
	(undef, $load_one) = split(':',$load_one);
	my $load_five = $uptime[4];

	$report{"load_one"} = $load_one;
	$report{"load_five"} = $load_five;

	server_auth_rate(\%report);
	system("mv " . WORKING . "circe.log.current " . WORKING . "circe.log.old") ;
	
	# if system has sustained load > 75% for the last five minutes,
	# report it
	if ($load_one > .75*$ncpu && $load_five > .75*$ncpu ) { 
		# This assumes a well-threaded load
		LibNAC::nac_err("System sustaining high load for more than 5 minutes.");
		LibNAC::nac_err("Current load is $load_one");
		server_collect_ps_data(\%report);
		server_test_ipfw(\%report);
		$report{"ngctl_create"} = server_test_ngctl_create_response();
		$report{"ngctl_destroy"} = server_test_ngctl_destroy_response();
		$report{"arp"} = server_test_arp_response();
		$report{"long"} = 'true';
	}
	# If the system is sustaining a load spike >= 90%, report it
	elsif ($load_one >= .9*$ncpu) { 
		LibNAC::nac_err("System experiencing extreme load,".
		" current average $load_one");
		server_collect_ps_data(\%report);
		server_test_ipfw(\%report);
		$report{"ngctl_create"} = server_test_ngctl_create_response();
		$report{"ngctl_destroy"} = server_test_ngctl_destroy_response();
		$report{"arp"} = server_test_arp_response();
		$report{"long"} = 'true';
	}
	server_produce_report(\%report);

}

sub server_auth_rate {
	my $old_log = WORKING . "circe.log.old";
	my $new_log = WORKING . "circe.log.current";
	my $href_report = shift;

	my $log_change = `diff $new_log $old_log | grep 'Context Change'`;
	$log_change =~ s/[><]//g;

	my $auth_count = 0;
	my $unauth_count = 0;

	my @entries = split("\n",$log_change);
	foreach my $log_line (@entries) {
		if ($log_line =~ m/registered/) {
			$auth_count += 1;
		} elsif ($log_line =~ m/unauth/) {
			$unauth_count +=1;
		}
	}
	$href_report->{"auth_count"} = $auth_count;
	$href_report->{"unauth_count"} = $unauth_count;
	return 1;
}

sub server_produce_report {
	my $href_report = shift;

	my $date = `date`;

	open(REPORT, ">>" . OUTPUT);

	print REPORT "Summary report for $date";
	print REPORT "Load Averages: 1: " . $href_report->{"load_one"} .
		" 5: " . $href_report->{"load_five"} . "\n";
	print REPORT "In the last minute:\n\t" . $href_report->{"auth_count"}
		. " users authenticated, " . $href_report->{"unauth_count"} . 
		" users unauthenticated.\n";

	if ($href_report->{"long"} eq 'true') {
		print REPORT "High load detected. Additional Information:\n";
		print REPORT "System Load Data:\n";
		print REPORT "Apache (httpd):\n";
		print REPORT "\tCPU: " . $href_report->{"apache_cpu"} . "\n";
		print REPORT "\tMEM: " . $href_report->{"apache_mem"} . "\n";
		print REPORT "CGI (perl):\n";
		print REPORT "\tCPU: " . $href_report->{"perl_cpu"} . "\n";
		print REPORT "\tMEM: " . $href_report->{"perl_mem"} . "\n";
		print REPORT "Squid:\n";
		print REPORT "\tCPU: " . $href_report->{"squid_cpu"} . "\n";
		print REPORT "\tMEM: " . $href_report->{"squid_mem"} . "\n";
		print REPORT "DHCPd:\n";
		print REPORT "\tCPU: " . $href_report->{"dhcp_cpu"} . "\n";
		print REPORT "\tMEM: " . $href_report->{"dhcp_mem"} . "\n";
		print REPORT "Circe:\n";
		print REPORT "\tCPU: " . $href_report->{"circe_cpu"} . "\n";
		print REPORT "\tMEM: " . $href_report->{"circe_mem"} . "\n";
		print REPORT "IPFW:\n";
		print REPORT "\tCPU: " . $href_report->{"ipfw_cpu"} . "\n";
		print REPORT "\tMEM: " . $href_report->{"ipfw_mem"} . "\n";
		print REPORT "ifconfig:\n";
		print REPORT "\tCPU: " . $href_report->{"ifconfig_cpu"} . "\n";
		print REPORT "\tMEM: " . $href_report->{"ifconfig_mem"} . "\n";

		print REPORT "IPFW health status:\n";
		if (($href_report->{"lockfile_10"}) eq 'true') {
			print REPORT "The following lockfiles found with age greater" .
				" than 10 seconds.\n";
			print REPORT "This indicates a likelihood of context change" . 
				" timeouts.\n";
			my $aref_lockfiles = $href_report->{"lockfile_10"};
			foreach my $lockfile (@$aref_lockfiles) { 
				print REPORT "$lockfile ";
			}
			print REPORT "\n";

			if (($href_report->{"lockfile_60"}) eq 'true') {
				print REPORT "The following lockfiles were found with age greater"
					. " than 1 minute.\n";
				print REPORT "This indicates a high likelihood that the" .
					" context change operation which produced this lockfile has"
					. " failed.\n";
				my $aref_lockfiles = $href_report->{"lockfile_age_60"};
				foreach my $lockfile (@$aref_lockfiles) {
					print REPORT "$lockfile ";
				}
				print REPORT "\n";
				
				if (($href_report->{"lockfile_300"}) eq 'true') {
					print REPORT "The following lockfiles were found with age"
						. " greater than 5 minutes.\n";
					print REPORT "This indicates that the context change" .
						" operation which produced this lockfile has failed." 
						. " These ip addresses have almost certainly become "
						. "unusable.\n";
					my $aref_lockfiles = $href_report->{"lockfile_age_300"};
					foreach my $lockfile (@$aref_lockfiles) {
						print REPORT "$lockfile ";
					}
					print REPORT "\n";
				}
			}
		} else {
			print REPORT "Lockfiles appear healthy.\n";
		}
		
		print REPORT "IPFW response time: " . 
			$href_report->{"ipfw_rule_modify_time"} . "\n";

		print REPORT "NGCTL Response Times:\n\tCreate: " . 
			$href_report->{"ngctl_create"} . "\n\tDestroy: " .
			$href_report->{"ngctl_destroy"} . "\n";

		print REPORT "ARP Response Time: " . $href_report->{"arp"} .
			"\n";

		print REPORT "IPFW rule activity rates: \n";
		if ($href_report->{"use_v6"}) {
			print REPORT "\tIPv6: " . $href_report->{"ipv6_pps"} . " packets, " .
				$href_report->{"ipv6_bps"} . " bytes\n";
		}

		print REPORT "\tAuthenticated incoming traffic: " . 
			$href_report->{"auth_incoming_pps"} . " packets, " .
			$href_report->{"auth_incoming_bps"} . " bytes\n";
		print REPORT "\tAuthenticated outgoing traffic: " . 
			$href_report->{"auth_outgoing_pps"} . " packets, " .
			$href_report->{"auth_outgoing_bps"} . " bytes\n";
		print REPORT "\tBandwidth incoming traffic: " . 
			$href_report->{"bandwidth_incoming_pps"} . " packets, " .
			$href_report->{"bandwidth_incoming_bps"} . " bytes\n";
		print REPORT "\tBandwidth outgoing traffic: " . 
			$href_report->{"bandwidth_outgoing_pps"} . " packets, " .
			$href_report->{"bandwidth_outgoing_bps"} . " bytes\n";
		print REPORT "\tSecurity incoming traffic: " . 
			$href_report->{"security_incoming_pps"} . " packets, " .
			$href_report->{"security_incoming_bps"} . " bytes\n";
		print REPORT "\tSecurity outgoing traffic: " . 
			$href_report->{"security_outgoing_pps"} . " packets, " .
			$href_report->{"security_outgoing_bps"} . " bytes\n";
		print REPORT "\tAdmin incoming traffic: " . 
			$href_report->{"admin_incoming_pps"} . " packets, " .
			$href_report->{"admin_incoming_bps"} . " bytes\n";
		print REPORT "\tAdmin outgoing traffic: " . 
			$href_report->{"admin_outgoing_pps"} . " packets, " .
			$href_report->{"admin_outgoing_bps"} . " bytes\n";
	} 

	print REPORT "End of Report.\n";
	close(REPORT);
}
sub server_collect_ps_data {
	my $href_report = shift;
	# Collect data on squid
	my $squid_cpu, $squid_mem;
	my @squid_processes = split("\n",
		`ps auxww | grep squid | grep -v grep | awk '{print \$3, \$4}'`);
	foreach my $process (@squid_processes) {
		my ($cpu, $mem) = split(" ",$process);
		$squid_cpu += $cpu;
		$squid_mem += $mem;
	}
	$href_report->{"squid_cpu"} = $squid_cpu;
	$href_report->{"squid_mem"} = $squid_mem;

	# Collect data on apache
	my $apache_cpu, $apache_mem;
	my @apache_processes = split("\n",
		`ps auxww | grep httpd | grep -v grep | awk '{print \$3, \$4}'`);
	foreach my $process (@apache_processes) {
		my ($cpu, $mem) = split (" ",$process);
		$apache_cpu += $cpu;
		$apache_mem += $mem;
	}
	$href_report->{"apache_cpu"} = $apache_cpu;
	$href_report->{"apache_mem"} = $apache_mem;

	# Collect data on ipfw
	my $ipfw_cpu, $ipfw_mem;
	my @ipfw_processes = split("\n",
		`ps auxww | grep httpd | grep -v grep | awk '{print \$3, \$4}'`);
	foreach my $process (@ipfw_processes) {
		my ($cpu, $mem) = split (" ",$process);
		$ipfw_cpu += $cpu;
		$ipfw_mem += $mem;
	}
	$href_report->{"ipfw_cpu"} = $ipfw_cpu;
	$href_report->{"ipfw_mem"} = $ipfw_mem;

	# Collect data on ifconfig
	my $ifconfig_cpu, $ifconfig_mem;
	my @ifconfig_processes = split("\n",
		`ps auxww | grep httpd | grep -v grep | awk '{print \$3, \$4}'`);
	foreach my $process (@ifconfig_processes) {
		my ($cpu, $mem) = split (" ",$process);
		$ifconfig_cpu += $cpu;
		$ifconfig_mem += $mem;
	}
	$href_report->{"ifconfig_cpu"} = $ifconfig_cpu;
	$href_report->{"ifconfig_mem"} = $ifconfig_mem;

	# Collect data on cgi processes (running perl)
	my $perl_cpu, $perl_mem;
	my @perl_processes = split("\n",
		`ps auxww | grep perl | grep -v grep | awk '{print \$3, \$4}'`);
	foreach my $process (@perl_processes) {
		my ($cpu, $mem) = split (" ",$process);
		$perl_cpu += $cpu;
		$perl_mem += $mem;
	}
	$href_report->{"perl_cpu"} = $perl_cpu;
	$href_report->{"perl_mem"} = $perl_mem;

	# Collect data on circe processes 
	my $circe_cpu, $circe_mem;
	my @circe_processes = split("\n",
		`ps auxww | grep circe | grep -v grep | awk '{print \$3, \$4}'`);
	foreach my $process (@circe_processes) {
		my ($cpu, $mem) = split (" ",$process);
		$circe_cpu += $cpu;
		$circe_mem += $mem;
	}
	$href_report->{"circe_cpu"} = $circe_cpu;
	$href_report->{"circe_mem"} = $circe_mem;

	# Collect data on dhcp
	my $dhcp_cpu, $dhcp_mem;
	my @dhcp_processes = split("\n",
		`ps auxww | grep dhcpd | grep -v grep | awk '{print \$3, \$4}'`);
	foreach my $process (@dhcp_processes) { 
		my ($cpu, $mem) = split(" ",$process); 
		$dhcp_cpu += $cpu;
		$dhcp_mem += $mem;
	}
	$href_report->{"dhcp_cpu"} = $dhcp_cpu;
	$href_report->{"dhcp_mem"} = $dhcp_mem;

	return 1;

}

sub server_test_ipfw {
	my $href_report = shift;
	my $aref_old_count = server_ipfw_count(undef,undef);
	# Test turnaround time for ipfw lookup
	$href_report->{"ipfw_rule_modify_time"} = server_test_ipfw_response();
	server_check_lockfiles($href_report);
	# Check counters on select ipfw rules
	sleep(5);
	server_ipfw_count($aref_old_count,$href_report);
	return 1;
}

sub server_test_ipfw_response {
	my $begin = time();
	`ipfw add 65534 deny ip from any to any`;
	`ipfw delete 65534`;
	my $duration = time() - $begin;
	return $duration;
}

sub server_test_arp_response {
	my $begin = time();
	`arp -s 10.0.0.254 00:aa:bb:cc:dd:ee pub`;
	`arp -d 10.0.0.254`;
	return (time() - $begin);
}

sub server_test_ngctl_create_response {
	my $begin = time();
	
	try {
		LibNAC::nac_ngctl("mkpeer ipfw: nat 299000000 out") || die;
		LibNAC::nac_ngctl("name ipfw:299000000 TestNAT000000") || die;
		LibNAC::nac_ngctl("connect ipfw: TestNAT000000: 199000000 in") || die;
		LibNAC::nac_ngctl("msg TESTNAT000000: setaliasaddr 127.1.1.1") || die;
		LibNAC::nac_ngctl("msg TESTNAT000000: redirectaddr {local_addr=127.1.1.1 alias_addr=127.1.1.1}") || die;
	} catch {
		LibNAC::nac_err("Error in LoadTest::Server: Failed to create ng_nat");
		return "Test FAILED: Error in ngctl creation";
	}
	return (time()-$begin);
}

sub server_test_ngctl_destroy_response {
	my $begin = time();
	
	LibNAC::nac_ngctl("shutdown ipfw:299000000");

	return (time() - $begin);
}



	
sub server_check_lockfiles {
	my $href_report = shift;
	# Check for existance and persistance of locks
	my @lockfile_age_10, @lockfile_age_60, @lockfile_age_300;
	my @lock_timestamps = split("\n",
	`ls -alD "%s" /var/run | grep nac.lock | grep -v nac.locks | awk '{print \$6,\$7}'`);
	my $date = `date +"%s"`;
	foreach my $lock (@lock_timestamps) {
		my ($timestamp, $lockfile) = split(" ",$lock);
		# This will break Jan 19, 2038 
		my $duration = $timestamp - $date;
		if ($duration > 10 ) {
			LibNAC::nac_err("Lock $lockfile is older than 10 seconds,"
				. " timeout is likely");
			push (@lockfile_age_10,$lockfile);
			$href_report->{"lockfile_10"} = 'true';
			if ($duration > 60 ) {
				LibNAC::nac_err("Lock $lockfile is older than 1 minute, unexpected"
					. " context change failure likely");
				$href_report->{"lockfile_60"} = 'true';
				push (@lockfile_age_60, $lockfile);
				if ($duration > 300) {
					LibNAC::nac_err("Lock $lockfile is older than 5 minutes,"
					. " ip address has probably become unusable");
					push (@lockfile_age_300, $lockfile);
					$href_report->{"lockfile_300"} = 'true';
				}
			}
		}
	}
	$href_report->{"lockfile_age_10"} = \@lockfile_age_10;
	$href_report->{"lockfile_age_60"} = \@lockfile_age_60;
	$href_report->{"lockfile_age_300"} = \@lockfile_age_300;
	return 1;
}
sub server_ipfw_count {
	# This subroutine counts the number of packets reported by ipfw 
	# for the first 14 circe rules. This requires root-level priviledges
	my ($aref_previous_count, $href_report) = @_;
	my @previous_count = @$aref_previous_count;
	my @ipfw_counts = split("\n",
		`ipfw show | head -n 9 | awk '{print \$2,\$3}'`); 
	my @packets, @bytes;
	foreach my $count (@ipfw_counts) {
		my ($packet, $byte) = split(" ",$count);
		push (@packets, $packet);
		push (@bytes, $byte);
	}
	my $count = push (@packets,time());
	push (@bytes, time());
	my @current_count = ([@packets],[@bytes]);

	if (defined ($aref_previous_count)) {
		# If the array reference exists then this represents the second
		# function call in the series
		my %confhash = LibNAC::nac_readconf();
		my $duration = time() - $previous_count[0][$count-1];
		@previous_count = @$aref_previous_count;
		my $i = 0;
		if ($confhash{"USE_V6"}) {
			$href_report->{"ipv6_pps"} = ($current_count[0][0] -
				$previous_count[0][0]) / $duration;
			$href_report->{"ipv6_bps"} = ($current_count[1][0] -
				$previous_count[1][0]) / $duration;
			$href_report->{"use_v6"} = 1;
			$i++;
		}

		$href_report->{"auth_incoming_pps"} = ($current_count[0][$i] -
			$previous_count[0][$i]) / $duration;
		$href_report->{"auth_incoming_bps"} = ($current_count[1][$i] - 
			$previous_count[1][$i]) / $duration;
		$i++;
		$href_report->{"auth_outgoing_pps"} = ($current_count[0][$i] - 
			$previous_count[0][$i]) / $duration;
		$href_report->{"auth_outgoing_bps"} = ($current_count[1][$i] -
			$previous_count[1][$i]) / $duration;
		$i++;
		$href_report->{"bandwidth_incoming_pps"} = ($current_count[0][$i] -
			$previous_count[0][$i]) / $duration;
		$href_report->{"bandwidth_incoming_bps"} = ($current_count[1][$i] -
			$previous_count[1][$i]) / $duration;
		$i++;
		$href_report->{"bandwidth_outgoing_pps"} = ($current_count[0][$i] -
			$previous_count[0][$i]) / $duration;
		$href_report->{"bandwidth_outgoing_bps"} = ($current_count[1][$i] -
			$previous_count[1][$i]) / $duration;
		$i++;
		$href_report->{"security_incoming_pps"} = ($current_count[0][$i] -
			$previous_count[0][$i]) / $duration;
		$href_report->{"security_incoming_bps"} = ($current_count[1][$i] -
			$previous_count[1][$i]) / $duration;
		$i++;
		$href_report->{"security_outgoing_pps"} = ($current_count[0][$i] -
			$previous_count[0][$i]) / $duration;
		$href_report->{"security_outgoing_bps"} = ($current_count[1][$i] -
			$previous_count[1][$i]) / $duration;
		$i++;
		$href_report->{"admin_incoming_pps"} = ($current_count[0][$i] -
			$previous_count[0][$i]) / $duration;
		$href_report->{"admin_incoming_bps"} = ($current_count[1][$i] -
			$previous_count[1][$i]) / $duration;
		$i++;
		$href_report->{"admin_outgoing_pps"} = ($current_count[0][$i] -
			$previous_count[0][$i]) / $duration;
		$href_report->{"admin_outgoing_bps"} = ($current_count[1][$i] -
			$previous_count[1][$i]) / $duration;
	}

	return \@current_count;
}

1;
