# Library LoadTest::Client
# This is the http client library for load testing circe captive portal boxes.
# The goal is to create a lightweight client to simulate normal
# user actions. In addition, the client should be able to simulate
# "bad" users- that is, those that act sporadically and unpredictably.
# This is a library implementation of the proof of concept script 
# http-client.pl
#
#

package LoadTest::Client;

use LWP::UserAgent;
use Statistics::Descriptive;
use Time::HiRes qw( time);
use constant {
	SUCCESS	=> 0,
	IPINUSE	=> -1,
	ERROR	=> -2,
	TIMEOUT => -3,
};
use LoadTest::Management;

#### Main Client Script ####

# This subroutine represents a single client with behavior defined by the
# arguments passed. The client may exhibit several behaviors: it may refresh
# the login page; it may login with incorrect credentials; and it may login
# correctly. All these behaviors occur a variable number of times. Additionally,
# the client remains logged in and sleeping for a user-configurable amount
# of time. At the conclusion of these operations, the client logs out.

sub client {
	my ($aref_client_net, $refresh_count, $failed_login_count,
		$login_count, $aref_client_login,$sleep_period) = @_;

	my $refresh_data = Statistics::Descriptive::Full->new();
	my $failed_logon_data = Statistics::Descriptive::Full->new();
	my $logon_data = Statistics::Descriptive::Full->new();
	my %metrics;
	$metrics{"private_ip"} = @{$aref_client_net}[0];
	

	# Begin with refresh behavior
	for (my $i = 0; $i < $refresh_count; $i++) {
		my $start = time();
		my $status = client_refresh($aref_client_net);
		my $duration = time()-$start;
		$refresh_data->add_data($duration);
		if ($status) {
			LoadTest::Management::management_log( "Server failed to respond 
				to refresh request: $status","err");
		}

	}
	# Prepare metrics for reporting
	if ($refresh_count >= 1) {
		my @refresh_data = $refresh_data->get_data();
		$metrics{"raw_refresh"} = \@refresh_data;
		$metrics{"avg_refresh"} = $refresh_data->mean();
		$metrics{"stddev_refresh"} = $refresh_data->standard_deviation();
		$metrics{"range_refresh"} = $refresh_data->sample_range();
	}


	# Now simulate failed logons
	for (my $i = 0; $i < $failed_login_count; $i++) {
		my $start = time();
		client_failed_login($aref_client_net);
		my $duration = time()-$start;
		$failed_logon_data->add_data($duration);
	}
	if ($failed_login_count >=1) { 
		my @failed_logon_data = $failed_logon_data->get_data();
		$metrics{"raw_failed_logon"} = \@failed_logon_data;
		$metrics{"avg_failed_logon"} = $failed_logon_data->mean();
		$metrics{"stddev_failed_logon"} 
			= $failed_logon_data->standard_deviation();
		$metrics{"range_failed_logon"} = $failed_logon_data->sample_range();
	}



	# Login with correct credentials
	for (my $i =0; $i < $login_count; $i++) {
		my $start = time();
		my $response = client_login($aref_client_net, $aref_client_login);
		my $duration = time()-$start;
		$logon_data->add_data($duration);
		if ($i == 0) { # First run case: check to make sure login succeeded
			unless ($response->content =~ m/Authentication complete./) {
				my @net=@$aref_client_net;
				LoadTest::Management::management_log("ERR: Client " . $net[0] .
					" unexpectedly failed to login successfully on first attempt"
					, "err");
					print "Client " . $net[0] . " failed to login successfully!\n";
				if ($response->content =~ m/Your guest account username/) {
					LoadTest::Management::management_log("ERR: Client login
						information for " . $net[0] . " is incorrect", "crit");
					$i = $login_count;
					$metrics{"logon_errors"} = "Client Login Incorrect";
				} elsif ($response->content =~ m/It looks like you've already/){
					LoadTest::Management::management_log("ERR: Client " . 
						$net[0] . " appears to have an existing session",
						"err");
					$i = $login_count;
					$metrics{"logon_errors"} = "Existing Session Conflicts";
				} else {
					LoadTest::Management::management_log("Error unknown.", 
						"err");
					$metrics{"logon_errors"} = "Unknown";
				}
			}
		}
	}
	if ($login_count >= 1) {
		my @logon_data = $logon_data->get_data();
		$metrics{"raw_logon"} = \@logon_data;
		$metrics{"avg_logon"} = $logon_data->mean();
		$metrics{"stddev_logon"} = $logon_data->standard_deviation();
		$metrics{"range_logon"} = $logon_data->sample_range();
	}

	# Stay dormant for a period of time
	sleep($sleep_period);

	# End the session
	my $response = client_logout($aref_client_net);
	if (!$response->is_success()) {
		my $status = $response->status_line();
		LoadTest::Management::management_log("Server failed to respond to refresh"
			. "request: $status","err");
	} else {
		# Check the output to ensure the client logged out cleanly
		unless ($response->content =~ m/You are now logged out/) {
			my @net=@$aref_client_net;
			LoadTest::Management::management_log("Client " . $net[0] . 
				" unexpectedly failed to logout","err");
			print "Client " . $net[0] . " failed to log out successfully!\n";
			if ($response->content =~ m/It looks like you haven't logged in/){
				LoadTest::Management::management_log("ERR: Captive portal cannot"
					. " find session for client " . $net[0], "err"); 
				$metrics{"logout_errors"} = "Not Logged In or Session Store " . 
					"Inconsistent";
			} else {
				LoadTest::Management::management_log("ERR: Client " . $net[0]
					. " experienced an unknown error. Check server-side logs",
					"err");
				$metrics{"logout_errors"} = "Unknown";
			}
		}
	}
	return \%metrics;
}


#### Client Behaviors ####

# Simulate refresh behavior. This simulates pure load on the http
# server by constantly querying for the login form without doing
# anything with it
# Expected behavior: Return the guest login screen
sub client_refresh {
	my $aref_client_net = @_;
	my ($priv_ip, $pub_ip, $mac) = @$aref_client_net;

	my $ua = LWP::UserAgent->new;
	$ua->agent("Circe-Test-Client/0.1");
	$ua->timeout(30);

	my $req = HTTP::Request->new(GET => "https://wlan.berkeley.edu/cgi-bin/login/guest.cgi?submit=Guest+Account&debug=true&privip=$priv_ip&pubip=$pub_ip&clientmac=$mac&url=http://www.google.com/");

	my $response = $ua->request($req);

	unless ($response->is_success) {
		return $response->status_line;
	} 
	return 0;
}

# Simulate improper credentials on login
# Expected behavior: Error, with opportunity to relogin
sub client_failed_login {
	my $aref_client_net = @_;
	my ($priv_ip, $pub_ip, $mac) = @$aref_client_net;
	my $ua = LWP::UserAgent->new;
	$ua->agent("Circe-Test-Client/0.1");
	$ua->timeout(30);
	$response=$ua->post(
		"https://wlan.berkeley.edu/cgi-bin/login/guest.cgi?submit=Guest+Account&debug=true&privip=$priv_ip&pubip=$pub_ip&clientmac=$mac&url=http://www.google.com/",
		[
			'username' => 'guest-1111',
			'password' => 'password',,
			'submit' => 'Login'
		],
	);
}

# Login for real. This value needs to be >=1 if the client should eventually
# successfully authenticate.
# Expected Behavior: successful login on first attempt, followed by ip
# in use error
sub client_login {
	my ($aref_client_net, $aref_client_login) = @_;
	my ($priv_ip, $pub_ip, $mac) = @$aref_client_net;
	my ($username, $password) = @$aref_client_login;
	my $ua = LWP::UserAgent->new;
	$ua->agent("Circe-Test-Client/0.1");
	$ua->timeout(30);
	$response=$ua->post(
		"https://wlan.berkeley.edu/cgi-bin/login/guest.cgi?submit=Guest+Account&debug=true&privip=$priv_ip&pubip=$pub_ip&clientmac=$mac&url=http://www.google.com/",
		[
			'username' => $username,
			'password' => $password,
			'submit' => 'Login'
		],
	);
	return $response;
}

# Logout and end session. Generally in the load-test environment we want
# session times and logouts to be deterministic. Therefore, we won't rely
# on manage-sessions to harvest these arp entry-less psuedo-clients.
# Instead, we will logout using the query string to identify each client.
sub client_logout {
	my ($aref_client_net) = @_;
	my ($priv_ip, $pub_ip, $mac) = @$aref_client_net;

    my $ua = LWP::UserAgent->new;
    $ua->agent("Circe-Test-Client/0.1");
    $ua->timeout(30);
	my $req = HTTP::Request->new(GET => "https://wlan.berkeley.edu/cgi-bin/logout/logout.cgi?submit=Guest+Account&debug=true&privip=$priv_ip&pubip=$pub_ip&clientmac=$mac&url=http://www.google.com/");

    my $response = $ua->request($req);

	return $response;
}


1;
